@@ -64,9 +64,11 @@ namespace Banter
 		protected ConnectionInfo connInfo;
 		protected IConnection tlpConnection;
 		protected IConnectionManager connManager;
-		protected IChannelGroup localInvitationGroup;
-		protected IChannelGroup remoteInvitationGroup;
-		protected IChannelGroup subscribedGroup;
+		
+		// Telepathy Groups
+		protected IChannelGroup publishGroup;
+		protected IChannelGroup subscribeGroup;
+		protected IChannelGroup denyGroup;
 
 		// Connected Handlers
 		protected bool aliasConnected = false;
@@ -74,6 +76,7 @@ namespace Banter
 		protected bool channelConnected = false;
 		protected bool presenceConnected = false;
 		protected bool messagingConnected = false;
+		protected bool capabilitiesConnected = false;
 		
 		// Supported interfaces
 		protected bool aliasing = false;
@@ -191,7 +194,7 @@ namespace Banter
 					return 
 						tlpConnection.InspectHandles (
 							HandleType.Contact,
-							localInvitationGroup.LocalPendingMembers);
+							publishGroup.LocalPendingMembers);
 				} catch{}
 				return null;
 			}
@@ -201,7 +204,7 @@ namespace Banter
 		{
 			get
 			{
-				try {return localInvitationGroup.LocalPendingMembers;} catch{}
+				try {return publishGroup.LocalPendingMembers;} catch{}
 				return null;
 			}
 		}
@@ -214,7 +217,7 @@ namespace Banter
 					return 
 						tlpConnection.InspectHandles (
 							HandleType.Contact,
-							remoteInvitationGroup.RemotePendingMembers);
+							subscribeGroup.RemotePendingMembers);
 				} catch{}
 				return null;
 			}
@@ -224,7 +227,7 @@ namespace Banter
 		{
 			get
 			{
-				try {return remoteInvitationGroup.RemotePendingMembers;} catch{}
+				try {return subscribeGroup.RemotePendingMembers;} catch{}
 				return null;
 			}
 		}
@@ -245,9 +248,54 @@ namespace Banter
 		/// </summary>
 		public Account (IConnection connection)
 		{
+			tlpConnection = connection;
 		}
 		#endregion
 
+		/// <summary>
+		/// Method to add or update a user to the provider list
+		/// </summary>
+		private
+		ProviderUser
+		AddProviderUser (uint id, string jid, ProviderUserRelationship relationship)
+		{
+			string key = ProviderUserManager.CreateKey (jid, this.protocol);
+			ProviderUser providerUser = null;
+			
+			try {
+				providerUser = ProviderUserManager.GetProviderUser (key);
+				if (providerUser != null) {
+					providerUser.TlpConnection = tlpConnection;
+					providerUser.AccountName = this.Name;
+					providerUser.Protocol = this.Protocol;
+					providerUser.ID = id;
+					providerUser.Relationship = relationship;
+				}
+			} 
+			catch (Exception fff) {
+				Logger.Debug ("Failed to get ProviderUser {0}", key);
+				Logger.Debug (fff.Message);
+			}
+				
+			if (providerUser == null) {
+				try {
+					providerUser =
+						ProviderUserManager.CreateProviderUser (
+							jid, 
+							this.protocol, 
+							relationship);
+					providerUser.TlpConnection = tlpConnection;
+					providerUser.AccountName = this.Name;
+					providerUser.ID = id;
+					Logger.Debug ("Member: {0} ID: {1} - object created", jid, id);
+				} catch{}
+			} else {
+				Logger.Debug ("Member: {0} ID: {1} - object updated", jid, id);
+			}
+			
+			return providerUser;
+		}
+
 		/// <summary>
 		/// This method should be called immediately
 		/// after a successful connect as other private
@@ -298,6 +346,7 @@ namespace Banter
 			
 			string[] pubArgs = {"publish"};
 			string[] subArgs = {"subscribe"};
+			string[] blockedArgs = {"deny"};
 			ObjectPath objectPath;
 			
 			try {
@@ -310,10 +359,29 @@ namespace Banter
 						pubHandles[0], 
 						true);
 						
-				localInvitationGroup = 
+				publishGroup = 
 					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
-				localInvitationGroup.MembersChanged += OnPublishedMembersChanged;
-				
+				publishGroup.MembersChanged += OnPublishedMembersChanged;
+			
+				uint groupFlags = publishGroup.GroupFlags;
+				Logger.Debug ("Group Flags: {0}", groupFlags);
+				if ((((uint) groupFlags & (uint) ChannelGroupFlag.CanAdd) == (uint) ChannelGroupFlag.CanAdd))
+					Logger.Debug ("  can add");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.CanRemove) == (uint) ChannelGroupFlag.CanRemove)
+					Logger.Debug ("  can remove");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.CanRescind) == (uint) ChannelGroupFlag.CanRescind)
+					Logger.Debug ("  can rescind");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageAdd) == (uint) ChannelGroupFlag.MessageAdd)
+					Logger.Debug ("  can add messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageRemove) == (uint) ChannelGroupFlag.MessageRemove)
+					Logger.Debug ("  can remove messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageAccept) == (uint) ChannelGroupFlag.MessageAccept)
+					Logger.Debug ("  can accept messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageReject) == (uint) ChannelGroupFlag.MessageReject)
+					Logger.Debug ("  can reject messages");
+				if (((uint) groupFlags & (uint) ChannelGroupFlag.MessageRescind) == (uint) ChannelGroupFlag.MessageRescind)
+					Logger.Debug ("  can rescind messages");
+					
 				// Next setup a channel to the subscribe list
 				uint[] subHandles = tlpConnection.RequestHandles (HandleType.List, subArgs);
 				objectPath = 
@@ -323,11 +391,24 @@ namespace Banter
 						subHandles[0], 
 						true);
 						
-				remoteInvitationGroup = 
+				subscribeGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+				subscribeGroup.MembersChanged += OnSubscribedMembersChanged;
+				
+				// Next setup a channel to the blocked list
+				uint[] blockedHandles = 
+					tlpConnection.RequestHandles (HandleType.List, blockedArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						blockedHandles[0], 
+						true);
+						
+				denyGroup = 
 					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
 					
-				subscribedGroup = remoteInvitationGroup;
-				subscribedGroup.MembersChanged += OnSubscribedMembersChanged;
+				denyGroup.MembersChanged += OnBlockedMembersChanged;
 				
 			} catch (Exception sgc) {
 				Logger.Debug ("Failed setting up group channels");
@@ -457,138 +538,91 @@ namespace Banter
 			string key;
 			
 			// First add any sent invitation users
-			Logger.Debug ("Adding SentInvitation users");
 			uint[] invitedHandles = this.SentInvitationHandles;
 			if (invitedHandles != null && invitedHandles.Length > 0) {
+				Logger.Debug ("Adding SentInvitation users");
 				string[] invitedUsers = this.SentInvitations;
-				
-				for (int i = 0; i < invitedUsers.Length; i++) {
-					
-					Logger.Debug ("  {0}", invitedUsers[i]);
-					
-					// update the provider user objects
-					key = ProviderUserManager.CreateKey (invitedUsers[i], protocol);
-					providerUser = null;
-					try {
-						providerUser = ProviderUserManager.GetProviderUser (key);
-						providerUser.TlpConnection = tlpConnection;
-						providerUser.AccountName = this.Name;
-						providerUser.Protocol = this.Protocol;
-						providerUser.ID = invitedHandles[i];
-						providerUser.Relationship = ProviderUserRelationship.SentInvitation;
-					} 
-					catch (Exception fff) {
-						Logger.Debug ("Failed to get ProviderUser {0}", key);
-						Logger.Debug (fff.Message);
-					}
-						
-					if (providerUser == null) {
-						try {
-							providerUser =
-								ProviderUserManager.CreateProviderUser (
-									invitedUsers[i], 
-									this.protocol, 
-									ProviderUserRelationship.SentInvitation);
-							providerUser.TlpConnection = tlpConnection;
-							providerUser.AccountName = this.Name;
-							providerUser.ID = invitedHandles[i];
-						} catch{}
-					}
-				}
+				for (int i = 0; i < invitedUsers.Length; i++)
+					AddProviderUser (
+						invitedHandles[i], 
+						invitedUsers[i],
+						ProviderUserRelationship.SentInvitation);
 			}
 			
 			// Next add received invitation users
 			uint[] inviteHandles = this.ReceivedInvitationHandles;
 			if (inviteHandles != null && inviteHandles.Length > 0) {
+				Logger.Debug ("Adding ReceivedInvitation users");
 				string[] inviteUsers = this.ReceivedInvitations;
-				
-				for (int i = 0; i < inviteUsers.Length; i++) {
-					
-					// update the provider user objects
-					key = ProviderUserManager.CreateKey (inviteUsers[i], protocol);
-							
-					providerUser = null;
-					try {
-						providerUser = ProviderUserManager.GetProviderUser (key);
-						providerUser.TlpConnection = tlpConnection;
-						providerUser.AccountName = this.Name;
-						providerUser.Protocol = this.Protocol;
-						providerUser.ID = inviteHandles[i];
-						providerUser.Relationship = ProviderUserRelationship.ReceivedInvitation;
-					} 
-					catch (Exception fff) {
-						Logger.Debug ("Failed to get ProviderUser {0}", key);
-						Logger.Debug (fff.Message);
-					}
-						
-					if (providerUser == null) {
-						try {
-							providerUser =
-								ProviderUserManager.CreateProviderUser (
-									inviteUsers[i], 
-									this.protocol, 
-									ProviderUserRelationship.ReceivedInvitation);
-							providerUser.AccountName = this.Name;
-							providerUser.TlpConnection = tlpConnection;
-							providerUser.ID = inviteHandles[i];
-						} catch{}
-					}
-				}
+				for (int i = 0; i < inviteUsers.Length; i++)
+					AddProviderUser (
+						inviteHandles[i], 
+						inviteUsers[i],
+						ProviderUserRelationship.ReceivedInvitation);
 			}
 			
-			// Add subscribed members			
-			string[] members = 
-				tlpConnection.InspectHandles (HandleType.Contact, subscribedGroup.Members);
-			
-			string[] aliasNames = null;
-			if (aliasing == true) {
-				aliasNames = tlpConnection.RequestAliases (subscribedGroup.Members);
-				//Logger.Debug ("# returned aliases: {0}", aliasNames.Length);	
-			}	
-
-			//Logger.Debug ("# of known contacts: {0}", members.Length);
-			for (int i = 0; i < members.Length; i++) {
-				//Logger.Debug ("MemberID: {0} Member: {1}", cl.Members[i], members[i]);
+			if (subscribeGroup.Members.Length > 0) {
+				Logger.Debug ("Adding Subscribed users");
+				// Add subscribed members			
+				string[] members = 
+					tlpConnection.InspectHandles (HandleType.Contact, subscribeGroup.Members);
 				
-				// update the provider user objects
-				key = ProviderUserManager.CreateKey (members[i], protocol);
-				providerUser = null;
-				try {
-					providerUser = ProviderUserManager.GetProviderUser (key);
-					providerUser.TlpConnection = tlpConnection;
-					providerUser.AccountName = this.Name;
-					providerUser.Protocol = this.Protocol;
-					providerUser.ID = subscribedGroup.Members[i];
-					providerUser.Relationship = ProviderUserRelationship.Linked;
+				string[] aliasNames = null;
+				if (aliasing == true)
+					aliasNames = tlpConnection.RequestAliases (subscribeGroup.Members);
+				
+				CapabilityInfo[] caps = null;
+				if (this.capabilities == true) {
+					caps = tlpConnection.GetCapabilities (subscribeGroup.Members);
+				}
+
+				//Logger.Debug ("# of known contacts: {0}", members.Length);
+				for (int i = 0; i < members.Length; i++) {
+					providerUser =
+						AddProviderUser(
+							subscribeGroup.Members[i],
+							members[i],
+							ProviderUserRelationship.Linked);
+							
 					if (aliasing == true && aliasNames != null)
 						providerUser.Alias = aliasNames[i];
-				} 
-				catch (Exception fff) {
-					Logger.Debug ("Failed to get ProviderUser {0}", key);
-					Logger.Debug (fff.Message);
-				}
-					
-				if (providerUser == null) {
-					try {
-						providerUser =
-							ProviderUserManager.CreateProviderUser (
-								members[i], 
-								this.protocol, 
-								ProviderUserRelationship.Linked);
-								
-						providerUser.AccountName = this.Name;
-						providerUser.ID = subscribedGroup.Members[i];
-						providerUser.TlpConnection = tlpConnection;
-					
-						if (aliasing == true && aliasNames != null)
-							providerUser.Alias = aliasNames[i];
-					} catch{}
+
+					/*						
+					public struct CapabilityInfo
+					{
+						public uint ContactHandle;
+						public string ChannelType;
+						public CapabilityFlag GenericFlags;
+						public ChannelMediaCapability TypeSpecificFlags;
+					}
+					*/
+						
+					// Setup the remote contact's media capabilities
+					if (capabilities == true && caps != null && caps.Length > 0) {
+						foreach (CapabilityInfo info in caps) {
+							if (info.ContactHandle == subscribeGroup.Members[i]) {
+								if ((info.TypeSpecificFlags & ChannelMediaCapability.Audio) ==
+										ChannelMediaCapability.Audio)
+									providerUser.MediaCapability = 
+										Banter.MediaCapability.Audio;
+
+								if ((info.TypeSpecificFlags & ChannelMediaCapability.Video) ==
+										ChannelMediaCapability.Video)
+										
+									// If video is reported audio is assumed	
+									providerUser.MediaCapability = 
+										Banter.MediaCapability.Video |
+										Banter.MediaCapability.Audio;
+								break;
+							}
+						}
+					}
 				}
 			}
 			
 			ConnectHandlers ();
 			if (presence == true)
-				tlpConnection.RequestPresence (subscribedGroup.Members);
+				tlpConnection.RequestPresence (subscribeGroup.Members);
 		}
 		
 		private void ConnectHandlers ()
@@ -598,6 +632,11 @@ namespace Banter
 			this.tlpConnection.NewChannel += OnNewChannel;
 			channelConnected = true;
 			
+			if (capabilities == true) {
+				tlpConnection.CapabilitiesChanged += OnCapabilitiesChanged;
+				capabilitiesConnected = true;
+			}
+			
 			if (aliasing == true) {
 				tlpConnection.AliasesChanged += OnAliasesChanged;
 				aliasConnected = true;
@@ -619,6 +658,11 @@ namespace Banter
 			//Logger.Debug ("Account::DisconnectHandlers");
 			
 			try {
+				if (capabilitiesConnected == true) {
+					tlpConnection.CapabilitiesChanged -= OnCapabilitiesChanged;
+					capabilitiesConnected = false;
+				}
+				
 				if (avatarsConnected == true) {
 					tlpConnection.AvatarUpdated -= OnAvatarUpdated;
 					avatarsConnected = false;
@@ -669,6 +713,22 @@ namespace Banter
 				user.AvatarToken = token;
 		}
 		
+		/// <summary>
+		///	Telepathy callback when capabilities change for a contact
+		/// </summary>
+		private void OnCapabilitiesChanged (CapabilitiesChangedInfo[] caps)
+		{
+			ProviderUser user;
+			foreach (CapabilitiesChangedInfo cap in caps) {
+				user = ProviderUserManager.GetProviderUser (cap.ContactHandle);
+				
+				/*
+				if (user != null)
+					user.Alias = info.NewAlias;
+				*/	
+			}
+		}
+		
 		/// <summary>
 		/// Private method to convert telepathy presence information into
 		/// a Banter.Presence object.  The ProviderUser object is updated with
@@ -742,6 +802,52 @@ namespace Banter
 		}
 		
 
+		/// <summary>
+		/// Callback method when members are added or removed from
+		/// the blocked list
+		/// </summary>
+		private
+		void
+		OnBlockedMembersChanged(
+			string message,
+			uint[] added,
+			uint[] removed,
+			uint[] localPending,
+			uint[] remotePending,
+			uint actor,
+			uint reason)
+		{
+			Logger.Debug ("OnBlockedMembersChanged - called");
+			Logger.Debug ("  Message: {0}", message);
+			Logger.Debug ("  # added: {0}", added.Length);
+			foreach (uint handle in added)
+			{
+				Logger.Debug ("    contact id: {0}", handle);
+				
+				/*
+				ProviderUser user =
+					ProviderUserManager.GetProviderUser (handle);
+				user.Relationship = ProviderUserRelationship.Linked;
+				*/
+			}
+				
+			Logger.Debug ("  # removed: {0}", removed.Length);
+			foreach (uint handle in removed) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("    contact id: {0} - {1}", handles[0], names[0]);
+			}
+				
+			Logger.Debug ("Remote Pending List");
+			foreach (uint handle in remotePending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+			}
+		}
+		
 		/// <summary>
 		/// Callback method when members are added or removed from
 		/// the subscribed list
@@ -758,14 +864,105 @@ namespace Banter
 			uint reason)
 		{
 			Logger.Debug ("OnSubscribedMembersChanged - called");
-			Logger.Debug ("  Message: {0}", message);
-			Logger.Debug ("  # added: {0}", added.Length);
-			foreach (uint handle in added)
-				Logger.Debug ("    contact id: {0}", handle);
+			if (message != null && message != String.Empty)
+				Logger.Debug ("  Message: {0}", message);
 				
-			Logger.Debug ("  # removed: {0}", removed.Length);
-			foreach (uint handle in removed)
-				Logger.Debug ("    contact id: {0}", handle);
+			try {
+				Logger.Debug ("  # added: {0}", added.Length);
+				
+				if (added.Length > 0) {
+					ProviderUser providerUser;
+					
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, added);
+					Logger.Debug ("past getting jid names");
+					
+					string[] aliasNames = null;
+					
+					/*
+					if (aliasing == true)
+						aliasNames = tlpConnection.RequestAliases (added);
+					Logger.Debug ("past getting alias");
+					*/
+					
+					CapabilityInfo[] caps = null;
+					if (this.capabilities == true)
+						caps = tlpConnection.GetCapabilities (added);
+					Logger.Debug ("past get caps");
+					
+					for (int i = 0; i < added.Length; i++) {
+						Logger.Debug (
+							" adding contact id: {0}  name: {1}", 
+							added[i], 
+							names[i]);
+					
+						providerUser =
+							AddProviderUser(
+								added[i],
+								names[i],
+								ProviderUserRelationship.Linked);
+						if (providerUser != null) {
+							if (aliasNames != null && 
+								aliasNames.Length >= i+1 &&
+								aliasNames[i] != null &&
+								aliasNames[i] != String.Empty)
+									providerUser.Alias = aliasNames[i];
+									
+							// Setup the remote contact's media capabilities
+							if (capabilities == true && caps != null && caps.Length > 0) {
+								foreach (CapabilityInfo info in caps) {
+									if (info.ContactHandle == added[i]) {
+										if ((info.TypeSpecificFlags & ChannelMediaCapability.Audio) ==
+												ChannelMediaCapability.Audio)
+											providerUser.MediaCapability = 
+												Banter.MediaCapability.Audio;
+
+										if ((info.TypeSpecificFlags & ChannelMediaCapability.Video) ==
+												ChannelMediaCapability.Video)
+												
+											// If video is reported audio is assumed	
+											providerUser.MediaCapability = 
+												Banter.MediaCapability.Video |
+												Banter.MediaCapability.Audio;
+										break;
+									}
+								}
+							}
+						}	
+					}
+				}
+				
+				Logger.Debug ("  # removed: {0}", removed.Length);
+				if (removed.Length > 0) {
+					for (int i = 0; i < removed.Length; i++) {
+						Logger.Debug ("    contact id: {0}", removed[i]);
+						ProviderUserManager.RemoveProviderUser (removed[i], this.protocol);
+					}
+				}
+
+				Logger.Debug ("Remote Pending List");
+				if (remotePending.Length > 0) {
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, remotePending);
+						
+					for (int i = 0; i < names.Length; i++) {
+						Logger.Debug ("    contact id: {0} - {1}", remotePending[i], names[i]);
+					}
+				}
+				
+				/*
+				foreach (uint handle in remotePending) {
+					uint[] handles = {handle};
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, handles);
+					Logger.Debug ("   {0} - {1}", handle, names[0]);
+					//this.remoteInvitationGroup.RemoveMembers (handles, String.Empty);
+				}
+				*/
+			} catch (Exception osmc) {
+				Logger.Debug (osmc.Message);
+				Logger.Debug (osmc.StackTrace);
+			}
 		}
 		
 
@@ -786,13 +983,83 @@ namespace Banter
 		{
 			Logger.Debug ("OnPublishedMembersChanged - called");
 			Logger.Debug ("  Message: {0}", message);
-			Logger.Debug ("  # added: {0}", added.Length);
-			foreach (uint handle in added)
-				Logger.Debug ("    contact id: {0}", handle);
+			
+			try {
+				Logger.Debug ("  # added: {0}", added.Length);
 				
-			Logger.Debug ("  # removed: {0}", removed.Length);
-			foreach (uint handle in removed)
-				Logger.Debug ("    contact id: {0}", handle);
+				if (added.Length > 0) {
+					// Add newly added members
+					string[] aliasNames = null;
+					string[] names = 
+						tlpConnection.InspectHandles (HandleType.Contact, added);
+			
+					for (int i = 0; i < added.Length; i++) {
+						Logger.Debug (
+							" adding contact id: {0}  name: {1}", 
+							added[i], 
+							names[i]);
+					
+						ProviderUser user =
+							ProviderUserManager.GetProviderUser (added[i]);
+						if (user != null) {
+							user.Relationship = ProviderUserRelationship.Linked;
+							if (aliasNames != null && 
+								aliasNames.Length >= i+1 &&
+								aliasNames[i] != null &&
+								aliasNames[i] != String.Empty)
+									user.Alias = aliasNames[i];
+						}	
+					}
+				}
+			} catch (Exception ae) {
+				Logger.Debug (ae.Message);			
+			}
+				
+			if (removed.Length > 0) {
+				try {
+					Logger.Debug ("  # removed: {0}", removed.Length);
+					for (int i = 0; i < removed.Length; i++) {
+						Logger.Debug ("    contact id: {0}", removed[i]);
+						//ProviderUserManager.RemoveProviderUser (removed[i], this.protocol);
+					}
+					
+				} catch (Exception re) {
+					Logger.Debug (re.Message);
+				}
+			}
+				
+			Logger.Debug ("Local Pending List");
+			foreach (uint handle in localPending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+				
+				// Add him to the list	
+				ProviderUser providerUser =				
+					ProviderUserManager.CreateProviderUser (
+						names[0], 
+						this.protocol, 
+						ProviderUserRelationship.ReceivedInvitation);
+				providerUser.TlpConnection = tlpConnection;
+				providerUser.AccountName = this.Name;
+				providerUser.ID = handle;
+				
+				if (aliasing == true) {
+					string[] aliasNames = null;
+					aliasNames = tlpConnection.RequestAliases (handles);
+					if (aliasNames != null && aliasNames.Length >= 1)
+						providerUser.Alias = aliasNames[0];
+				}
+			}
+			
+			Logger.Debug ("Remote Pending List");
+			foreach (uint handle in remotePending) {
+				uint[] handles = {handle};
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, handles);
+				Logger.Debug ("   {0} - {1}", handle, names[0]);
+			}
 		}
 		
 		/// <summary>
@@ -821,173 +1088,6 @@ namespace Banter
 					suppressHandler);
 				
 			return;
-			
-			/*
-			Conversation conversation = null;
-			switch (channelType)
-			{
-				case org.freedesktop.Telepathy.ChannelType.Text:
-				{
-					if (handle == 0)
-						return;
-						
-					// Check if we have an existing conversation with the peer user
-					ProviderUser pu = null;
-					try {
-						pu = ProviderUserManager.GetProviderUser (handle);
-					} catch{}
-					
-					if (pu == null) return;
-					
-					if (ConversationManager.Exist (pu) == true) {
-						Logger.Debug ("An existing conversation with {0} already exists", pu.Uri);
-						return;
-					}
-					
-					Person peer = PersonManager.GetPersonByJabberId (pu.Uri);
-					ChatWindow cw = null;
-					
-					Logger.Debug ("Peer: {0}", peer.Id);
-					Logger.Debug ("Peer Name: {0}", peer.EDSContact.GivenName);
-					
-//					if (ChatWindow.AlreadyExist (peer.Id) == true) { 
-//						Logger.Debug ("ChatWindow already exists with this peer");
-//						ChatWindow.PresentWindow (peer.Id);
-//					} else {
-						try
-						{
-							Logger.Debug ("creating conversation object");
-							conversation = ConversationManager.Create (this, peer, false);
-							IChannelText txtChannel = 
-								Bus.Session.GetObject<IChannelText> (busName, channelPath);
-							
-							conversation.SetTextChannel (txtChannel);
-							Logger.Debug ("created new conversation object");
-						
-							//cw = new ChatWindow (conversation);
-							//cw.Present();
-						}
-						catch (Exception es)
-						{
-							Logger.Debug (es.Message);
-							Logger.Debug (es.StackTrace);
-						}
-					}
-					break;
-	//			}
-				
-				case org.freedesktop.Telepathy.ChannelType.StreamedMedia:
-				{
-					uint peerHandle;
-					
-					IChannelStreamedMedia ichannel = 
-						Bus.Session.GetObject<IChannelStreamedMedia> (
-							busName,
-							channelPath);
-					
-					if(ichannel.Members.Length > 0) {
-						foreach(uint ch in ichannel.Members) {
-							Logger.Debug("Member in ichannel.Members {0}", ch);
-						}
-
-					}
-					if(ichannel.Members.Length > 0) {
-						peerHandle = ichannel.Members[0];
-					}
-					else
-						return;
-					
-					if (handle == 0) {
-					
-						if (ichannel.LocalPendingMembers.Length > 0) {
-							Logger.Debug ("Incoming media conversation");
-							handle = ichannel.LocalPendingMembers[0];
-						} else if (ichannel.RemotePendingMembers.Length > 0) {
-							handle = ichannel.RemotePendingMembers[0];
-							Logger.Debug ("Pulled the handle from ichannel.RemotePendingMembers");
-							return;
-						} else if (ichannel.Members.Length > 0) {
-							handle = ichannel.Members[0];
-							Logger.Debug ("Pulled the handle from ichannel.Members");
-							return;
-						} else {
-							Logger.Debug ("Could not resolve the remote handle");
-							return;
-						}	
-					} else {
-						Logger.Debug ("Handle was non-zero {0} - returning", handle);
-						return;
-					}
-					
-					if (handle == this.tlpConnection.SelfHandle) {
-						Logger.Debug ("Handle was me - yay");
-						uint[] meHandles = {handle};
-						
-						uint[] ids = {ichannel.Members[0]};
-							
-						// Check if we have an existing conversation with the peer user
-						ProviderUser puMe = null;
-						ProviderUser puPeer = null;
-						
-						try {
-							puMe = ProviderUserManager.GetProviderUser (handle);
-							puPeer = ProviderUserManager.GetProviderUser(peerHandle);
-						} catch{}
-					
-						if (puMe == null) return;
-						if (puPeer == null) return;
-					
-					
-						if (ConversationManager.Exist (puPeer) == true) {
-							Logger.Debug ("An existing conversation with {0} already exists", puPeer.Uri);
-							return;
-						}
-
-						ichannel.AddMembers(meHandles, String.Empty);
-					
-						Person peer = PersonManager.GetPersonByJabberId (puPeer.Uri);
-						ChatWindow cw = null;
-					
-						Logger.Debug ("Peer: {0}", peer.Id);
-						Logger.Debug ("Peer Name: {0}", peer.DisplayName);
-					
-//						if (ChatWindow.AlreadyExist (peer.Id) == true) { 
-//							Logger.Debug ("ChatWindow already exists with this peer");
-//							ChatWindow.PresentWindow (peer.Id);
-//						} else {
-							try
-							{
-								Logger.Debug ("creating conversation object");
-								conversation = ConversationManager.Create (this, peer, false);
-								IChannelText txtChannel = 
-									Bus.Session.GetObject<IChannelText> (busName, channelPath);
-							
-								conversation.SetTextChannel (txtChannel);
-								conversation.SetMediaChannel (ichannel, channelPath);
-								Logger.Debug ("created new conversation object");
-								
-								//cw = new ChatWindow (conversation);
-								//cw.Present();
-								
-								conversation.SetPreviewWindow (cw.PreviewWindowId);
-								conversation.SetPeerWindow (cw.VideoWindowId);
-								conversation.StartVideo (false);
-							}
-							catch (Exception es)
-							{
-								Logger.Debug (es.Message);
-								Logger.Debug (es.StackTrace);
-							}
-//						}
-					}
-					
-					break;
-				}
-				
-				default:
-					break;
-			}
-			*/
 		}
 		
 		/// <summary>
@@ -995,11 +1095,11 @@ namespace Banter
 		/// </summary>
 		public void AddMember (uint id, string message)
 		{
-			if (this.localInvitationGroup == null)
+			if (this.publishGroup == null)
 				throw new ApplicationException ("Group instance unavailable");
 				
 			uint[] ids = {id};
-			this.localInvitationGroup.AddMembers (ids, message);	
+			this.publishGroup.AddMembers (ids, message);	
 		}
 
 
@@ -1010,32 +1110,69 @@ namespace Banter
 		/// </summary>
 		public void AuthorizeUser (bool authorize, uint id, string message)
 		{
-			if (this.localInvitationGroup == null)
+			if (this.publishGroup == null)
 				throw new ApplicationException ("Group instance unavailable");
 				
 			uint[] ids = {id};
 			if (authorize == true)
-				localInvitationGroup.AddMembers (ids, message);
+				publishGroup.AddMembers (ids, message);
 			else
-				localInvitationGroup.RemoveMembers (ids, message);
+				publishGroup.RemoveMembers (ids, message);
 		}
 
+		/// <summary>
+		/// Method to block an existing user
+		/// block == true blocks the user, false unblocks
+		/// </summary>
+		public void BlockUser (bool block, uint id, string message)
+		{
+			if (this.denyGroup == null)
+				throw new ApplicationException ("Group instance unavailable");
+				
+			uint[] ids = {id};
+			if (block == true) {
+				//subscribeGroup.RemoveMembers (ids, message);
+				denyGroup.AddMembers (ids, message);
+			} else {
+				denyGroup.RemoveMembers (ids, message);
+				//subscribeGroup.AddMembers (ids, message);
+			}	
+		}
+
+
 		/// <summary>
 		/// Method to invite a user to enable chat
 		/// </summary>
-		public void InviteUser (bool authorize, uint id, string message)
+		public void InviteUser (string username)
 		{
-			if (this.remoteInvitationGroup == null)
+			Logger.Debug ("InviteUser - called");
+			
+			if (this.subscribeGroup == null)
 				throw new ApplicationException ("Group instance unavailable");
+			
+			try {
+				string[] names = {username};
+				uint[] inviteHandles =
+					tlpConnection.RequestHandles (HandleType.Contact, names);
 				
+				subscribeGroup.AddMembers (inviteHandles, String.Empty);
 				
-			/*		
-			uint[] ids = {id};
-			if (authorize == true)
-				localInvitationGroup.AddMembers (ids, message);
-			else
-				localInvitationGroup.RemoveMembers (ids, message);
-			*/
+				string[] normalizedNames = 
+					tlpConnection.InspectHandles (HandleType.Contact, inviteHandles);
+
+				ProviderUser providerUser =				
+					ProviderUserManager.CreateProviderUser (
+						normalizedNames[0], 
+						this.protocol, 
+						ProviderUserRelationship.SentInvitation);
+				providerUser.TlpConnection = tlpConnection;
+				providerUser.AccountName = this.Name;
+				providerUser.ID = inviteHandles[0];
+				
+			} catch (Exception iu ) {
+				Logger.Debug (iu.Message);
+				Logger.Debug (iu.StackTrace);
+			}
 		}
 
 		/// <summary>
@@ -1043,11 +1180,75 @@ namespace Banter
 		/// </summary>
 		public void RemoveUser (uint id, string message)
 		{
-			if (this.subscribedGroup == null)
-				throw new ApplicationException ("Group instance unavailable");
+			Logger.Debug ("Account::RemoveUser - called");
+			Logger.Debug ("  removing: {0}", id);
+			
+			if (this.tlpConnection == null)
+				throw new ApplicationException ("Invalid telepathy connection");
 				
-			uint[] ids = {id};
-			this.subscribedGroup.RemoveMembers (ids, message);	
+			try {
+				ProviderUser providerUser =
+					ProviderUserManager.GetProviderUser (id);
+				if (providerUser == null)
+					throw new ApplicationException ("Not a valid user");
+					
+				uint[] ids = {id};
+				if (providerUser.Relationship == ProviderUserRelationship.ReceivedInvitation) {
+					if (this.publishGroup == null)
+						throw new ApplicationException ("Group doesn't exist");
+					Logger.Debug ("  removing member from the local invitation group");
+					publishGroup.RemoveMembers (ids, message);				
+				} else if (providerUser.Relationship == ProviderUserRelationship.Linked ||
+							providerUser.Relationship == ProviderUserRelationship.SentInvitation) {
+					if (this.subscribeGroup == null)
+						throw new ApplicationException ("Group doesn't exist");
+						
+					Logger.Debug ("  removing member from the subscribed group");
+					subscribeGroup.RemoveMembers (ids, message);				
+				}
+			
+			} catch (Exception ru) {
+				Logger.Debug ("Exception in RemoveUser");
+				Logger.Debug (ru.Message);
+			}
+				
+				
+			/*				
+//			string[] subArgs = {"subscribe"};
+			string[] subArgs = {"known"};
+			ObjectPath objectPath;
+			IChannelGroup subGroup;
+			
+			try {
+				uint[] subHandles = tlpConnection.RequestHandles (HandleType.List, subArgs);
+				objectPath = 
+					tlpConnection.RequestChannel (
+						org.freedesktop.Telepathy.ChannelType.ContactList, 
+						HandleType.List, 
+						subHandles[0], 
+						true);
+						
+				subGroup = 
+					Bus.Session.GetObject<IChannelGroup> (connInfo.BusName, objectPath);
+
+				Logger.Debug ("Members in \"Known\" group");					
+				string[] names = 
+					tlpConnection.InspectHandles (HandleType.Contact, subGroup.Members);
+				for (int i = 0; i < subGroup.Members.Length; i++) {
+					Logger.Debug ("Handle: {0}  Name: {1}", subGroup.Members[i], names[i]);
+				}
+					
+				subGroup.MembersChanged += OnSubscribedMembersChanged;
+				uint[] ids = {id};
+				subGroup.RemoveMembers (ids, message);				
+				subGroup.MembersChanged -= OnSubscribedMembersChanged;
+				subGroup = null;
+				
+			} catch (Exception ru) {
+				Logger.Debug ("Exception in RemoveUser");
+				Logger.Debug (ru.Message);
+			}
+			*/
 		}
 
 		/// <summary>
